@@ -3,7 +3,7 @@ import sys
 from io import BytesIO, StringIO
 from typing import TYPE_CHECKING
 
-import botocore
+from botocore.exceptions import ClientError
 
 from deci_common.abstractions.abstract_logger import ILogger
 from deci_common.aws_connection import AWSConnector
@@ -11,6 +11,18 @@ from deci_common.aws_connection import AWSConnector
 if TYPE_CHECKING:
     from typing import Any, Dict, List, Optional, Union
 
+    from mypy_boto3_s3 import S3Client, S3ServiceResource
+    from mypy_boto3_s3.type_defs import (
+        BucketLifecycleConfigurationTypeDef,
+        CopySourceTypeDef,
+        CreateBucketOutputTypeDef,
+        DeleteTypeDef,
+        ErrorTypeDef,
+        GetObjectOutputTypeDef,
+        HeadObjectOutputTypeDef,
+        LifecycleRuleTypeDef,
+    )
+
 
 class KeyNotExistInBucketError(Exception):
     pass
@@ -45,7 +57,7 @@ class S3Connector(ILogger):
         """
         try:
             self.s3_client.head_object(Bucket=self.bucket_name, Key=s3_key_to_check)
-        except botocore.exceptions.ClientError as ex:
+        except ClientError as ex:
             if ex.response["Error"]["Code"] == "404":
                 return False
             else:
@@ -56,7 +68,7 @@ class S3Connector(ILogger):
         else:
             return True
 
-    def get_object_by_etag(self, bucket_relative_file_name: str, etag: str) -> "Optional[object]":
+    def get_object_by_etag(self, bucket_relative_file_name: str, etag: str) -> "Optional[GetObjectOutputTypeDef]":
         """
         get_object_by_etag - Gets S3 object by its ETag header if it. exists
         :param bucket_relative_file_name: The name of the file in the bucket (relative)
@@ -67,16 +79,14 @@ class S3Connector(ILogger):
             etag = etag.strip('"')
             s3_object = self.s3_client.get_object(Bucket=self.bucket_name, Key=bucket_relative_file_name, IfMatch=etag)
             return s3_object
-        except botocore.exceptions.ClientError as ex:
-            if ex.response["Error"]["Code"] == "404":
-                return False
-            else:
+        except ClientError as ex:
+            if ex.response["Error"]["Code"] != "404":
                 self._logger.error(
                     "Failed to check ETag: " + str(etag) + " existence in bucket " + str(self.bucket_name)
                 )
         return None
 
-    def create_bucket(self, s3_client: "Any" = None) -> bool:
+    def create_bucket(self, s3_client: "Optional[S3Client]" = None) -> "CreateBucketOutputTypeDef":
         """
         Creates a bucket with the initialized bucket name.
         :return: The new bucket response
@@ -102,20 +112,20 @@ class S3Connector(ILogger):
             )
 
             return create_bucket_response
-        except botocore.exceptions.ClientError as err:
+        except ClientError as err:
             self._logger.fatal(f'Failed to create bucket "{self.bucket_name}": {err}')
             raise err
 
     def check_bucket_exists(
         self,
         bucket_name: "Optional[str]" = None,
-        s3_resource: "Any" = None,
+        s3_resource: "Optional[S3ServiceResource]" = None,
     ) -> bool:
         resource = s3_resource or self.s3_resource
         bucket_name = bucket_name if bucket_name else self.bucket_name
         return resource.Bucket(bucket_name) in resource.buckets.all()  # type: ignore[union-attr]
 
-    def delete_bucket(self, s3_resource: "Any" = None) -> bool:
+    def delete_bucket(self, s3_resource: "Optional[S3ServiceResource]" = None) -> bool:
         """
         Deletes a bucket with the initialized bucket name.
         :return: True if succeeded.
@@ -128,15 +138,15 @@ class S3Connector(ILogger):
             bucket.objects.all().delete()
             bucket.delete()
             self._logger.debug("Successfully Deleted Bucket: " + self.bucket_name + " from S3")
-        except botocore.exceptions.ClientError as ex:
+        except ClientError as ex:
             self._logger.fatal(f"Failed to delete bucket {self.bucket_name}: {ex}")
             raise ex
         return True
 
-    def get_object_metadata(self, s3_key: str) -> "Dict[str, Any]":
+    def get_object_metadata(self, s3_key: str) -> "HeadObjectOutputTypeDef":
         try:
             return self.s3_client.head_object(Bucket=self.bucket_name, Key=s3_key)
-        except botocore.exceptions.ClientError as ex:
+        except ClientError as ex:
             if ex.response["Error"]["Code"] == "404":
                 msg = "[" + sys._getframe().f_code.co_name + "] - Key does not exist in bucket)"
                 self._logger.error(msg)
@@ -152,7 +162,7 @@ class S3Connector(ILogger):
         try:
             self._logger.debug("Deleting Key: " + s3_key_to_delete + " from S3 bucket: " + self.bucket_name)
             obj_status = self.s3_client.head_object(Bucket=self.bucket_name, Key=s3_key_to_delete)
-        except botocore.exceptions.ClientError as ex:
+        except ClientError as ex:
             if ex.response["Error"]["Code"] == "404":
                 self._logger.error("[" + sys._getframe().f_code.co_name + "] - Key does not exist in bucket)")
             return False
@@ -254,7 +264,7 @@ class S3Connector(ILogger):
         try:
             self._logger.debug("Uploading Key: " + key_to_download + " from S3 bucket: " + self.bucket_name)
             self.s3_client.download_file(Bucket=self.bucket_name, Filename=target_path, Key=key_to_download)
-        except botocore.exceptions.ClientError as ex:
+        except ClientError as ex:
             if ex.response["Error"]["Code"] == "404":
                 self._logger.error("[" + sys._getframe().f_code.co_name + "] - Key does exist in bucket)")
             else:
@@ -291,9 +301,8 @@ class S3Connector(ILogger):
         page_iterator = paginator.paginate(Bucket=self.bucket_name, Prefix=prefix)
 
         for item in page_iterator.search("Contents"):
-            if item is not None:
-                if item["Key"] == s3_bucket_path_prefix:
-                    continue
+            if item is None or item["Key"] == s3_bucket_path_prefix:
+                continue
             key_to_download = item["Key"]
             local_filename = key_to_download.split("/")[-1]
             self.download_key(target_path=local_download_dir + "/" + local_filename, key_to_download=key_to_download)
@@ -312,7 +321,7 @@ class S3Connector(ILogger):
         self.download_key(target_path=local_download_dir + "/" + local_filename, key_to_download=s3_file_path)
         return local_filename
 
-    def empty_folder_content_by_path_prefix(self, s3_bucket_path_prefix: str) -> "List[Dict[str, Any]]":
+    def empty_folder_content_by_path_prefix(self, s3_bucket_path_prefix: str) -> "List[List[ErrorTypeDef]]":
         """
         empty_folder_content_by_path_prefix - Deletes all the files in the specified bucket path
             :param s3_bucket_path_prefix: The "folder" to empty
@@ -321,14 +330,14 @@ class S3Connector(ILogger):
         paginator = self.s3_client.get_paginator("list_objects")
         page_iterator = paginator.paginate(Bucket=self.bucket_name, Prefix=s3_bucket_path_prefix)
 
-        files_dict_to_delete: "Dict[str, Any]" = dict(Objects=[])
-        errors_list: "List[Dict[str, Any]]" = []
+        files_dict_to_delete: "DeleteTypeDef" = dict(Objects=[])
+        errors_list: "List[List[ErrorTypeDef]]" = []
 
         for item in page_iterator.search("Contents"):
             if item is not None:
                 if item["Key"] == s3_bucket_path_prefix:
                     continue
-                files_dict_to_delete["Objects"].append(dict(Key=item["Key"]))
+                files_dict_to_delete["Objects"] = [*files_dict_to_delete["Objects"], dict(Key=item["Key"])]
 
                 # IF OBJECTS LIMIT HAS BEEN REACHED, FLUSH
                 if len(files_dict_to_delete["Objects"]) >= 1000:
@@ -343,8 +352,8 @@ class S3Connector(ILogger):
 
     def _delete_files_left_in_list(
         self,
-        errors_list: "List[Dict[str, Any]]",
-        files_dict_to_delete: "Dict[str, Any]",
+        errors_list: "List[List[ErrorTypeDef]]",
+        files_dict_to_delete: "DeleteTypeDef",
     ) -> None:
         try:
             s3_response = self.s3_client.delete_objects(Bucket=self.bucket_name, Delete=files_dict_to_delete)
@@ -459,27 +468,31 @@ class S3Connector(ILogger):
         self._logger.info(
             f"Copying the bucket object {self.bucket_name}/{source_key} to {destination_bucket_name}/{destination_key}"
         )
-        copy_source = {"Bucket": self.bucket_name, "Key": source_key}
+        copy_source: "CopySourceTypeDef" = {"Bucket": self.bucket_name, "Key": source_key}
 
         # Copying the key
         bucket = self.s3_resource.Bucket(destination_bucket_name)  # type: ignore[union-attr]
         bucket.copy(copy_source, destination_key)
         return True
 
-    def change_lifecycle_configuration(self, lifecycle_configuration: "Dict[str, Any]") -> None:
+    def change_lifecycle_configuration(self, lifecycle_configuration: "BucketLifecycleConfigurationTypeDef") -> None:
         self.s3_client.put_bucket_lifecycle_configuration(
             Bucket=self.bucket_name, LifecycleConfiguration=lifecycle_configuration
         )
 
+    def add_lifecycle_configuration_rule(self, rule: "LifecycleRuleTypeDef") -> None:
+        lifecycle_config = self.s3_client.get_bucket_lifecycle_configuration(Bucket=self.bucket_name)
+        if "Rules" in lifecycle_config:
+            lifecycle_config["Rules"].append(rule)
+        else:
+            lifecycle_config["Rules"] = [rule]
+        self.change_lifecycle_configuration(lifecycle_configuration={"Rules": lifecycle_config["Rules"]})
+
     def set_expiration_for_prefix(self, prefix: str, days_to_expiration: int) -> None:
-        lifecycle_configuration = {
-            "Rules": [
-                {
-                    "Expiration": {"Days": days_to_expiration},
-                    "Filter": {"Prefix": prefix},
-                    "ID": f"Expiration for {self.bucket_name}/{prefix}",
-                    "Status": "Enabled",
-                }
-            ]
+        expiration_rule: "LifecycleRuleTypeDef" = {
+            "Expiration": {"Days": days_to_expiration},
+            "Filter": {"Prefix": prefix},
+            "ID": f"Expiration for {self.bucket_name}/{prefix}",
+            "Status": "Enabled",
         }
-        self.change_lifecycle_configuration(lifecycle_configuration=lifecycle_configuration)
+        self.add_lifecycle_configuration_rule(rule=expiration_rule)
