@@ -1,9 +1,8 @@
 from datetime import datetime
-from typing import Generic, TYPE_CHECKING
+from typing import TYPE_CHECKING
 from warnings import warn
 
 from deci_common.abstractions.abstract_logger import ILogger
-from deci_common.data_connection.mongo_connector import DocumentType
 
 if TYPE_CHECKING:
     from typing import Any, Dict, List, Optional
@@ -16,18 +15,18 @@ if TYPE_CHECKING:
     from deci_common.data_connection.mongo_connector import MongoConnector
 
 
-class MongoDBInterface(Generic[DocumentType], ILogger):
+class MongoDBInterface(ILogger):
     # FIXME: https://deci.atlassian.net/browse/PLFM-3408
     _CREATION_TIME = "creation_time"
     _UPDATE_TIME = "update_time"
 
-    def __init__(self, collection: str, mongo_connector: "MongoConnector[DocumentType]"):
+    def __init__(self, collection: str, mongo_connector: "MongoConnector"):
         super().__init__()
         self._collection = collection
         self._mongo_connector = mongo_connector
 
     @property
-    def collection(self) -> "Collection[DocumentType]":
+    def collection(self) -> "Collection[Dict[str, Any]]":
         return self._mongo_connector.db[self._collection]
 
     @property
@@ -60,21 +59,21 @@ class MongoDBInterface(Generic[DocumentType], ILogger):
         search_filter: "Dict[str, Any]",
         *,
         projection: "Dict[str, Any]" = {},
-    ) -> "Optional[DocumentType]":
+    ) -> "Optional[Dict[str, Any]]":
         """
         Find single element from DB with a specified search filter
         @param search_filter: Filter for the search
         @param projection: Projection to apply
-        @return: DocumentType
+        @return: Dict[str, Any]
         """
         return self.collection.find_one(filter=search_filter, projection=projection)
 
-    def find_by_id(self, _id: "Any", *, id_key: str = "id") -> "Optional[DocumentType]":
+    def find_by_id(self, _id: "Any", *, id_key: str = "id") -> "Optional[Dict[str, Any]]":
         """
         Find single element from DB by ID
         @param _id: ID value
         @param id_key: Name of field containing ID in DB. Optional, defaults to "id".
-        @return: DocumentType
+        @return: Dict[str, Any]
         """
         return self.find_one(search_filter={id_key: _id})
 
@@ -83,12 +82,12 @@ class MongoDBInterface(Generic[DocumentType], ILogger):
         search_filter: "Dict[str, Any]" = {},
         *,
         projection: "Dict[str, Any]" = {},
-    ) -> "Cursor[DocumentType]":
+    ) -> "Cursor[Dict[str, Any]]":
         """
         Find all elements from DB with a specified search filter
         @param search_filter: Filter for the search
         @param projection: Projection to apply
-        @return: Cursor[DocumentType]
+        @return: Cursor[Dict[str, Any]
         """
         return self.collection.find(filter=search_filter, projection=projection)
 
@@ -119,7 +118,7 @@ class MongoDBInterface(Generic[DocumentType], ILogger):
         updated_document.update(self._update_time_partial)
         return self._update(search_filter=search_filter, update={"$set": updated_document}, upsert=upsert)
 
-    def upsert(self, updated_document: "Dict[str, Any]", *, id_key: str = "id") -> "DocumentType":
+    def upsert(self, updated_document: "Dict[str, Any]", *, id_key: str = "id") -> "Dict[str, Any]":
         search_filter = {id_key: updated_document[id_key]}
         updated_document.update(self._update_time_partial)
         updated = self._update(search_filter=search_filter, update={"$set": updated_document}, upsert=False)
@@ -155,7 +154,7 @@ class MongoDBInterface(Generic[DocumentType], ILogger):
         )
         return update_result.modified_count != 0
 
-    def aggregate(self, pipeline: "List[Dict[str, Any]]") -> "CommandCursor[DocumentType]":
+    def aggregate(self, pipeline: "List[Dict[str, Any]]") -> "CommandCursor[Dict[str, Any]]":
         return self.collection.aggregate(pipeline=pipeline)
 
     def ping(self) -> "Dict[str, Any]":
