@@ -38,6 +38,7 @@ class HardwareType(str, Enum):
     SKYLAKE_SP = "Skylake-SP"
     CASCADE_LAKE_GCP = "Cascade Lake GCP"
     IMX8 = "NXP i.MX 8M mini"
+    C5_2XLARGE = "c5.2xlarge"
 
     HAILO8 = "Hailo-8"
     AMBARELLA = "Ambarella"
@@ -100,6 +101,7 @@ class HardwareMachineModel(str, Enum):
     SKYLAKE_SP = "m5.2xlarge"
     A100_40G = "a2-highgpu-1g"
     CASCADE_LAKE_GCP = "n2-standard-4"
+    C5_2XLARGE = "c5.2xlarge"
 
     NANO = HardwareType.NANO.value
     XAVIER = HardwareType.XAVIER.value
@@ -164,6 +166,7 @@ class MapHardwareTypeToFamily(str, Enum):
     SKYLAKE_SP = InferenceHardware.CPU.value
     CASCADE_LAKE_GCP = InferenceHardware.CPU.value
     IMX8 = InferenceHardware.CPU.value
+    C5_2XLARGE = InferenceHardware.CPU.value
 
 
 class InferyVersion(str, Enum):
@@ -204,6 +207,7 @@ class MapHardwareTypeToInferyVersion(str, Enum):
     SKYLAKE_SP = InferyVersion.CPU.value
     CASCADE_LAKE_GCP = InferyVersion.CPU.value
     IMX8 = InferyVersion.CPU.value
+    C5_2XLARGE = InferyVersion.CPU.value
 
 
 class HardwareEnvironment(str, Enum):
@@ -247,6 +251,7 @@ class HardwareTaint(str, Enum):
     SKYLAKE_SP = "intel.com/skylake-sp"
     CASCADE_LAKE_GCP = "intel.com/xeon"
     IMX8 = UNKNOWN
+    C5_2XLARGE = "intel.com/c5-2xlarge"
 
 
 class HardwareLabel(str, Enum):
@@ -277,6 +282,7 @@ class HardwareLabel(str, Enum):
     SKYLAKE_SP = "intel-skylake-sp"
     CASCADE_LAKE_GCP = "intel-xeon"
     IMX8 = UNKNOWN
+    C5_2XLARGE = "intel-c5-2xlarge"
 
 
 class MapHardwareTypeToVendor(str, Enum):
@@ -303,6 +309,7 @@ class MapHardwareTypeToVendor(str, Enum):
     NUC_TIGER_LAKE = HardwareVendor.INTEL.value
     SKYLAKE_SP = HardwareVendor.INTEL.value
     CASCADE_LAKE_GCP = HardwareVendor.INTEL.value
+    C5_2XLARGE = HardwareVendor.INTEL.value
     IMX8 = HardwareVendor.NXP.value
 
 
@@ -320,6 +327,7 @@ class MapHardwareTypeToEnvironment(str, Enum):
     Broadwell = HardwareEnvironment.AWS.value
     Icelake = HardwareEnvironment.AWS.value
     SKYLAKE_SP = HardwareEnvironment.AWS.value
+    C5_2XLARGE = HardwareEnvironment.AWS.value
 
     # GCP
     A100_40G = HardwareEnvironment.GCP.value
@@ -363,6 +371,7 @@ class MapHardwareTypeToDefaultBatchSizeList(list, Enum):  # type: ignore[type-ar
     SKYLAKE_SP = get_enum_values(BatchSize)
     CASCADE_LAKE_GCP = get_enum_values(BatchSize)
     IMX8 = get_enum_values(BatchSize)
+    C5_2XLARGE = get_enum_values(BatchSize)
 
 
 class HardwareImageRepository(str, Enum):
@@ -435,6 +444,7 @@ class MapHardwareTypeToGroup(str, Enum):
     Icelake = HardwareGroup.CPU.value
     SKYLAKE_SP = HardwareGroup.CPU.value
     CASCADE_LAKE_GCP = HardwareGroup.CPU.value
+    C5_2XLARGE = HardwareGroup.CPU.value
 
     APPLE_IPHONE_A11 = HardwareGroup.CONSUMER_EDGE.value
     APPLE_IPHONE_A12 = HardwareGroup.CONSUMER_EDGE.value
@@ -480,6 +490,7 @@ class HardwareTypeLabel(str, Enum):
     A100_80G = HardwareType.A100_80G.value
     NUC_TIGER_LAKE = HardwareType.NUC_TIGER_LAKE.value
     IMX8 = HardwareType.IMX8.value
+    C5_2XLARGE = HardwareType.C5_2XLARGE.value
     HAILO8 = HardwareType.HAILO8.value
     AMBARELLA = HardwareType.AMBARELLA.value
     APPLE_IPHONE_A11 = HardwareType.APPLE_IPHONE_A11.value
